<?php

class Delo extends Actioner
{
    const CHECK_SIGNATURE = 1;

    /**
     * Возвращает кусок xml с <result>
     * @param int $code 1 – Успех. Продолжить оплату. 2 – Неудача. Прервать процесс оплаты
     * @param string $desc описание
     * @return string xml
     */
    private function xmlResult($code, $desc)
    {
        return '<result><code>' . $code . '</code><desc><![CDATA[' . $desc . ']]></desc></result>';
    }

    /**
     * Возвращает простой ответ на запрос от платежного шлюза, простой это когда надо только ответить все ок или все плохо и почему
     * @param int $code 1 – Успех. Продолжить оплату. 2 – Неудача. Прервать процесс оплаты
     * @param string $message описание ошибки или описание успеха
     * @param string $request 'check' = первый запрос, 'register' = второй запрос
     * @return string xml ответа
     */
    private function createSimpleResponse($code, $message, $request = 'check')
    {
        if ('check' === $request) {
            $xml = '<payment-avail-response>';
        } else {
            $xml = '<register-payment-response>';
        }
        $xml .= $this->xmlResult($code, $message);
        if ('check' === $request) {
            $xml .= '</payment-avail-response>';
        } else {
            $xml .= '</register-payment-response>';
        }
        return $xml;
    }

    /**
     * Возвращает отрицательный ответ на запрос проверки платежа от платежного шлюза
     * @param int $code 1 – Успех. Продолжить оплату. 2 – Неудача. Прервать процесс оплаты
     * @param string $message описание ошибки или описание успеха
     * @param string $request 'check' = первый запрос, 'register' = второй запрос
     * @return string xml
     */
    private function createResponseFailed($error, $request = 'check')
    {
        if (empty($error)) {
            // Ошибка по умолчанию
            $error = 'Failed request';
        }
        return $this->createSimpleResponse(2, $error, $request);
    }

    /**
     * Проверка настроек, возвращает либо ошибки, либо null
     * @param array $paymetSettings
     * @return string|null
     */
    private function checkSettingsPayment($paymetSettings)
    {
        $errors = [];
        if (empty($paymetSettings['merch_id'])) {
            $errors[] = 'Empty setting "merch_id"';
        }
        if (empty($paymetSettings['purchase_account_amount_id'])) {
            $errors[] = 'Empty setting "purchase_account_amount_id"';
        }
        if (empty($paymetSettings['currency'])) {
            $errors[] = 'Empty setting "currency"';
        }
        // @todo: еще какие-нибудь проверки настроек
        if (!empty($errors)) {
            return implode(PHP_EOL, $errors);
        }
        return null;
    }

    /**
     * Возвращает настройки способа оплаты
     * @param int $paymentId id способа оплаты
     * @return array массив с настройками
     */
    private function getPaymentSettings($paymentId)
    {
        $q = 'SELECT
                `id`,
                `activity`,
                `paramArray`
            FROM `' . PREFIX . 'payment`
            WHERE `id` = ' . (int)$paymentId;
        $r = DB::query($q);
        $row = DB::fetchAssoc($r);
        $settings = [];
        $param = json_decode($row['paramArray']);
        if (!empty($param)) {
            foreach ($param as $key => $value) {
                if ($value != '') {
                    $value = CRYPT::mgDecrypt($value);
                }
                $settings[$key] = $value;
            }
        }
        return $settings;
    }

    /**
     * Выставляет заголовок и выплевывает xml
     * @param string $xml
     */
    private function responseXml($xml)
    {
        header('Content-type: application/xml');
        exit($xml);
    }

    /**
     * Преобразуем рубли в копейки
     * @param string $amount
     * @return int
     */
    private function rubToPennies($amount)
    {
        if (in_array(MG::getSetting('priceFormat'), ['1234','1 234','1,234'])) {
            $amount = round($amount);
        }
        $amount = number_format($amount, 2, '', '');
        return (int)$amount;
    }

    /**
     * Преобразуем копейки в рубли
     * @param string $amount
     * @return string
     */
    private function penniesToRub($amount)
    {
        return number_format($amount / 100, 2, '.', '');
    }

    /**
     * Возвращает положительный ответ на запрос проверки платежа от платежного шлюза
     * @param string $descResult Краткое описание результатов проверки. Не более 125 символов
     * @param string $descShortPurchase Краткое описание платежа. Не более 30 символов
     * @param string $descLongPurchase Полное описание платежа. Не более 125 символов. Это описание отображается клиенту на странице ввода реквизитов банковской карты
     * @param int $amount Сумма к оплате. В копейках
     * @param int $currency Код Валюты
     * @return string xml
     */
    private function createResponseSuccessfulCheck($descResult, $descShortPurchase, $descLongPurchase, $amount, $currency)
    {
        return '<payment-avail-response>' . $this->xmlResult(1, $descResult)
            . '<purchase>'
            . '<shortDesc><![CDATA[' . $descShortPurchase . ']]></shortDesc>'
            . '<longDesc><![CDATA[' . $descLongPurchase . ']]></longDesc>'
            . '<account-amount>'
            . '<amount>' . $amount . '</amount>'
            . '<currency>' . $currency . '</currency>'
            . '<exponent>2</exponent>'
            . '</account-amount>'
            . '</purchase>'
            . '</payment-avail-response>';
    }

    /**
     * Путь до сертификата
     * @return string
     */
    private function certPath()
    {
        $path = __DIR__ . '/cert/';
        if (isset($_SERVER['DELOBANK_ENVIRONMENT'])) {
            $path .= 'delobank_test.crt';
        } else {
            $path .= 'delobank_production.crt';
        }
        return $path;
    }

    /**
     * Проверка существования файла сертификата
     * @return bool
     */
    private function checkCertificateFile()
    {
        if (!self::CHECK_SIGNATURE) {
            return true;
        }
        $certFile = $this->certPath();
        if (!file_exists($certFile) || !is_file($certFile)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Выделяет подписанные данные из запроса
     * @return string часть url-а c доменом до &signature=
     */
    private function getDataFromUrl()
    {
        $url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on' ? 'https' : 'http') . "://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
        $pos = strpos($url, '&signature=');
        return substr($url, 0, $pos);
    }

    /**
     * Проверка подписи (ЭЦП) в запросе от эквайера
     * @return bool true - если проверка пройдена и false если не пройдена по какой-либо причине
     */
    private function checkSignature()
    {
        if (!self::CHECK_SIGNATURE) {
            // Если проверка ЦП выключена, параметра signature не будет, проверку ЦП не делаем всегда ок
            return true;
        }
        $signature = base64_decode($_GET['signature']);
        $data = $this->getDataFromUrl();
        $certFile = $this->certPath();
        // Получаем из сертификата публичный ключ для проверки
        $public_key = openssl_pkey_get_public('file://' . $certFile);
        if ($public_key === false) {
            return false;
        }
        // Проверяем подпись и освобождаем ключ
        $signature_check = openssl_verify($data, $signature, $public_key, OPENSSL_ALGO_SHA1);
        openssl_free_key($public_key);
        if ($signature_check !== 1) {
            return false;
        }
        return true;
    }

    /**
     * Возвращает сообщение ошибки по ее коду
     * @param string $code
     * @return string
     */
    private function getErrorMessageByCode($code)
    {
        $errors = [
            '-2' => 'Отказ в проведении платежа',
            '-3' => 'Ошибка при проведении платежа',
            '-4' => 'Успешный платеж отменен по инициативе магазина на этапе фиксации платежа',
            '2'  => 'Магазин отказался проводить платеж на этапе проверки',
            '4'  => 'Произошла ошибка при взаимодействии с магазином на этапе проверки платежа',
            '21' => 'Проверка платежа в магазине не выполнялась. Возможно магазин, или биллинг были заблокированы',
            '53' => 'Транзакция завершилась по таймауту, так как пользователь по каким-то причинам отказался продолжать платеж',
        ];
        if (isset($errors[$code])) {
            return $errors[$code];
        }
        return 'Ошибка проведения платежа с кодом ошибки: ' . $code;
    }

    /**
     * Обработка первого запроса от платежного шлюза
     */
    public function check()
    {
        /**
         * Проверяем наличие входных параметров
         */
        if (empty($_GET['o_payment_id']) || empty($_GET['o_order_id']) || empty($_GET['merch_id']) || empty($_GET['trx_id'])) {
            $this->responseXml($this->createResponseFailed('Failed request'));
        }
        $paymentId = (int)$_GET['o_payment_id'];
        $paymentSettings = $this->getPaymentSettings($paymentId);
        /**
         * Проверяем настройки
         */
        $errors = $this->checkSettingsPayment($paymentSettings);
        if (!empty($errors)) {
            $this->responseXml($this->createResponseFailed($errors));
        }
        /**
         * Проверяем merch_id
         */
        if ($paymentSettings['merch_id'] != $_GET['merch_id']) {
            $this->responseXml($this->createResponseFailed('Not valid merch_id'));
        }
        /**
         * Проверяем id заказа
         */
        $orderId = (int)$_GET['o_order_id'];
        if (empty($orderId)) {
            $this->responseXml($this->createResponseFailed('Not valid order_id'));
        }
        if (!$this->checkCertificateFile()) {
            $this->responseXml($this->createResponseFailed('Certificate file error'));
        }
        /**
         * Проверим существование заказа и что его стоимость не равна нулю
         */
        $orderModel = new Models_Order;
        $orders = $orderModel->getOrder('id = ' . (int)$orderId);
        if (empty($orders[$orderId])) {
            $this->responseXml($this->createResponseFailed('Order not exist'));
        }
        if (empty($orders[$orderId]['summ'])) {
            $this->responseXml($this->createResponseFailed('Order total sum null'));
        }
        $sum = $this->rubToPennies($orders[$orderId]['summ']);
        // @todo: на основе заказа получить информацию о товарах
        // @todo: передавать информацию о покупателе доп. тегах - ФИО, телефон, email
        $response = $this->createResponseSuccessfulCheck(
            'Проверка платежа заказа № ' . $orders[$orderId]['number'],
            'Оплата заказа №' . $orders[$orderId]['number'],
            'Оплата заказа №' . $orders[$orderId]['number'] . ' ' . $_SERVER['HTTP_HOST'],
            $sum,
            $paymentSettings['currency']
        );
        $this->responseXml($response);
    }

    /**
     * Обработка второго запроса от платежного шлюза, прием статуса платежа, обновление статуса заказа
     */
    public function register()
    {
        /**
         * Проверяем наличие входных параметров
         */
        if (empty($_GET['o_payment_id']) || empty($_GET['o_order_id']) || empty($_GET['merch_id']) || empty($_GET['trx_id']) || empty($_GET['signature']) || empty($_GET['amount'])) {
            $this->responseXml($this->createResponseFailed('Failed request', 'register'));
        }
        $paymentId = (int)$_GET['o_payment_id'];
        $paymentSettings = $this->getPaymentSettings($paymentId);
        /**
         * Проверяем настройки
         */
        $errors = $this->checkSettingsPayment($paymentSettings);
        if (!empty($errors)) {
            $this->responseXml($this->createResponseFailed($errors, 'register'));
        }
        /**
         * Проверяем merch_id
         */
        if ($paymentSettings['merch_id'] != $_GET['merch_id']) {
            $this->responseXml($this->createResponseFailed('Not valid merch_id', 'register'));
        }
        /**
         * Проверяем id заказа
         */
        $orderId = (int)$_GET['o_order_id'];
        if (empty($orderId)) {
            $this->responseXml($this->createResponseFailed('Not valid order_id', 'register'));
        }
        /**
         * Проверяем ЦП
         */
        if (!$this->checkCertificateFile()) {
            $this->responseXml($this->createResponseFailed('Certificate file error', 'register'));
        }
        if (!$this->checkSignature()) {
            $this->responseXml($this->createResponseFailed('Check Signature failed', 'register'));
        }
        /**
         * Проверим существование заказа
         */
        $orderModel = new Models_Order;
        $orders = $orderModel->getOrder('id = ' . (int)$orderId);
        if (empty($orders[$orderId])) {
            $this->responseXml($this->createResponseFailed('Order not exist', 'register'));
        }
        /**
         * Проверяем сколько оплатили
         */
        $sum = $this->rubToPennies($orders[$orderId]['summ']);
        $amount = (int)$_GET['amount'];
        if ($amount != $sum) {
            $this->responseXml($this->createResponseFailed('Order sum failed', 'register'));
        }
        $amount = $this->penniesToRub($amount);
        $resultCode = (int)$_GET['result_code'];
        if ($resultCode != 1) {
            // Платежный шлюз отвечает, что что-то не так, мы отвечаем, что инфо принято, но со статусом заказа ничего не делаем
            $error = $this->getErrorMessageByCode((string)((int)$_GET['ext_result_code']));
            $this->responseXml($this->createSimpleResponse(1, $error, 'register'));
        }
        // Установка статуса заказу оплачен
        $paymentModel = new Controllers_Payment();
        $paramForPaided = [
            'paymentOrderId' => $orderId,
            'paymentAmount'  => $amount,
            'paymentID'      => $amount,
        ];
        $paymentModel->actionWhenPayment($paramForPaided);
        $this->responseXml($this->createSimpleResponse(1, 'Заказ № ' . $orderId . ' - успех', 'register'));
    }
}
